@@ -10,6 +10,7 @@
 ** destructor.
 *****************************************************************************/
 /*
+
 segatex written by Shintaro Fujiwara.
 This code is dedicated to my daughter, mone.
 */
@@ -54,13 +55,81 @@ int segatex_set_time_all_real();
 void segatex_set_time();
 void segatex_set_time_all();
 void segatex_yum_update();
-void segatex_load_from_policy();
+void segatex_module_list();
 void segatex_load_from_my_policy();
+void segatex_remove_my_policy();
 void segatex_write_to_my_policy();
 void segatex_show_yum_update();
 void segatex_show_interface();
 
-void segatex::segatex_load_from_policy()
+void segatex::segatex_remove_my_policy()
+{
+        using namespace std;
+        #define PWD_BUF 8600
+        const char* remove_file="./remove_echo";
+        ofstream os;
+        os.open(remove_file);
+        if(os){
+                FILE *read_fp;
+                #define SIZE_TMP 8192
+                char buffer[SIZE_TMP];
+                int chars_read;
+                memset(buffer, '\0', sizeof(buffer));
+                read_fp = popen("semodule -r my_policy_by_segatex", "r");
+                if(read_fp != NULL){
+                        chars_read = fread(buffer, sizeof(char), SIZE_TMP, read_fp);
+                        while(chars_read>0){
+                                buffer[chars_read -1] = '\0';
+                                //os << "Reading:-" << endl;
+                                os << buffer;
+                                chars_read= fread(buffer, sizeof(char), SIZE_TMP, read_fp);
+                        }
+                        pclose(read_fp);
+                }
+                os.close();
+                QFile file( remove_file ); // Read the text from a file
+                if ( file.open( IO_ReadOnly ) ) {
+                        QTextStream stream( &file );
+                        textEdit2->setText( stream.read() );
+                }
+		status_label_1->setText("Please click module_list button and check policy has been removed !");
+        }
+}
+
+void segatex::segatex_module_list()
+{
+        using namespace std;
+        #define PWD_BUF 8600
+        const char* module_list_file="./module_list";
+        ofstream os;
+        os.open(module_list_file);
+        if(os){
+                FILE *read_fp;
+                #define SIZE_TMP 8192
+                char buffer[SIZE_TMP];
+                int chars_read;
+                memset(buffer, '\0', sizeof(buffer));
+                read_fp = popen("semodule -l", "r");
+                if(read_fp != NULL){
+                        chars_read = fread(buffer, sizeof(char), SIZE_TMP, read_fp);
+                        while(chars_read>0){
+                                buffer[chars_read -1] = '\0';
+                                //os << "Reading:-" << endl;
+                                os << buffer;
+                                chars_read= fread(buffer, sizeof(char), SIZE_TMP, read_fp);
+                        }
+                        pclose(read_fp);
+                }
+                os.close();
+                QFile file( module_list_file ); // Read the text from a file
+                if ( file.open( IO_ReadOnly ) ) {
+                        QTextStream stream( &file );
+                        textEdit2->setText( stream.read() );
+                }
+        }
+}
+
+void segatex::segatex_load_from_my_policy()
 {
 	using namespace std;
         const char* filename_policy="./my_policy_by_segatex.te";
@@ -78,40 +147,58 @@ void segatex::segatex_load_from_policy()
 	}
 }
 
-void segatex::segatex_load_from_my_policy()
-{
-	using namespace std;
-        const char* filename_policy="my_data/my_policy_by_segatex.te";
-	ifstream is;
-	is.open(filename_policy);
-	if(is){
-		QFile file( filename_policy ); 
-		if ( file.open( IO_ReadOnly ) ) {
-			QTextStream stream( &file );
-			textEdit2->setText( stream.read() );
-		}
-		is.close();
-	}else{
-		;//do nothing
-	}
-}
-
 void segatex::segatex_write_to_my_policy()
 {
 	using namespace std;
-	const char* my_policy_file="./my_data/my_policy_by_segatex.te";
-	ofstream fs;
-	fs.open(my_policy_file);
-	if(fs){
-		QFile file( my_policy_file ); 
-		if ( file.open( IO_ReadWrite ) ) {
-			QTextStream stream( &file );
-        		stream << textEdit2->text();
-			textEdit2->setModified( FALSE );
+	const char* my_policy_file="./my_policy_by_segatex.te";
+	const char* load_my_policy_file="./load_my_policy";
+	ifstream os_pre;
+	os_pre.open(my_policy_file);
+	if(os_pre){
+		os_pre.close();
+		ofstream os;
+		os.open(load_my_policy_file);
+		if(os){
+			//int check=chdir(my_data_dir);
+			int check=0;
+			if(check==0){
+				FILE *read_fp_install;
+				FILE *read_fp;
+				#define SIZE_TMP 8192
+				char buffer[SIZE_TMP];
+				int chars_read;
+				memset(buffer, '\0', sizeof(buffer));
+				read_fp_install = popen("make -f /usr/share/selinux/devel/Makefile", "r");
+				if(read_fp_install != NULL){
+					chars_read = fread(buffer, sizeof(char), SIZE_TMP, read_fp_install);
+					while(chars_read>0){
+						buffer[chars_read -1] = '\0';
+						//os << "Reading:-" << endl;
+						os << buffer;
+						chars_read= fread(buffer, sizeof(char), SIZE_TMP, read_fp_install);
+					}
+					pclose(read_fp_install);
+				}
+				read_fp = popen("semodule -i my_policy_by_segatex.pp", "r");
+				if(read_fp != NULL){
+					chars_read = fread(buffer, sizeof(char), SIZE_TMP, read_fp);
+					while(chars_read>0){
+						buffer[chars_read -1] = '\0';
+						//os << "Reading:-" << endl;
+						os << buffer;
+						chars_read= fread(buffer, sizeof(char), SIZE_TMP, read_fp);
+					}
+					pclose(read_fp);
+				}
+			}
+			os.close();
+			QFile file( load_my_policy_file ); // Read the text from a file
+				if ( file.open( IO_ReadOnly ) ) {
+				QTextStream stream( &file );
+				textEdit2->setText( stream.read() );
+			}
+		status_label_1->setText("Please click module_list button and check policy has been installed !");
 		}
-		fs.close();
-	}else{
-		;//do nothing
 	}
 }
 void segatex::segatex_show_yum_update()
@@ -260,7 +347,7 @@ void segatex::segatex_show_interface()
 	using namespace std;
 	using namespace boost;
 	const char* filename="/var/log/audit/audit.log";
-	//const char* filename_test="selpaw_interface.csv_test";
+	const char* filename_my_log="./audit_log_mine_must_be_here";
         const char* filename_policy="./my_policy_by_segatex.te";
 	if(log_state == 0){
 		log_from_time_int = segatex::segatex_set_time_all_real(); 
@@ -278,7 +365,8 @@ void segatex::segatex_show_interface()
         ofstream os_engine_test;
 	ofstream os_policy;
         is_engine.open(filename_engine);
-        //os_engine_test.open(filename_test);
+	fstream os_my_log(filename_my_log,ios::in | ios::app);
+	fstream os_my_log_2(filename_my_log, ios::app);
 	//When selpaw_interface.csv is missing.
         if(!is_engine){
 		os_policy.open(filename_policy);
@@ -324,20 +412,98 @@ void segatex::segatex_show_interface()
 
 			vector<string> v2;
 			vector<string>::iterator p2_v, p2_v_end;
+			vector<string> v3;
+			vector<string>::iterator p3_v, p3_v_end;
 			multimap<string ,string>::iterator p2,p2_end;
 			multimap<string ,string>::iterator p2_m,p2_m_end;
-			os_policy.open(filename_policy);
-			if(os_policy)
-			{
-				char buffer[MAX_LOG];
+			//write to my_log
+			char buffer[MAX_LOG];
+
+			//os_my_logҤ餤ơǸιԤunixtimeintѴѿƤ
+			os_my_log.open(filename_my_log);
+			if(os_my_log){
+				//deniedåѤ
+				do{
+					os_my_log.getline(buffer, sizeof buffer);
+					v3.push_back(buffer);
+				}while(!os_my_log.eof());
+			}else{
+				;//do nothing
+			}
+			os_my_log.close();
+			string u_time_str = "";
+			for(p3_v=v3.begin(); p3_v<v3.end()-1;p3_v++){
+				u_time_str = *p3_v;
+				//cout << u_time_str << endl;
+			}
+			const char* u_time = "\\([0-9]{10}";
+			//cout << u_time_str << endl;
+			const char* u_time_char_pre = u_time_str.c_str();
+			regex u_time_exp(u_time);
+			cmatch what;
+			int u_time_int=0;
+			int u_time_present_int=0;
+			if(regex_search(u_time_char_pre,what,u_time_exp)){
+				string u_time_match = what[0];
+				string blank_str="";
+				u_time_match=u_time_match.replace(0,1,blank_str);
+				//cout << u_time_match << endl;
+				const char* u_time_char = u_time_match.c_str();
+				u_time_int= atoi(u_time_char);
+				//cout << u_time_int << endl;
+			}else{
+				//cout << "no-match" << endl;
+			}
+
+			os_my_log_2.open(filename_my_log);
+			if(os_my_log_2){
 				const char* words="denied";
 				//deniedåѤ
 				do{
 					is.getline(buffer, sizeof buffer);
 					if(strstr(buffer,words)){
-						v.push_back(buffer);
+						const char* buffer_const_char= buffer;
+						//⤷u_time_int礭Сos_my_logɵƤ
+						if(regex_search(buffer_const_char,what,u_time_exp)){
+							string u_time_match_present = what[0];
+							//cout << u_time_match_present << endl;
+							string blank_str="";
+							u_time_match_present=u_time_match_present.replace(0,1,blank_str);
+							const char* u_time_match_present_char = u_time_match_present.c_str();
+							u_time_present_int= atoi(u_time_match_present_char);
+							if(u_time_present_int > u_time_int){
+								//cout << "u_time_present_int" << u_time_present_int << endl;
+								//cout << "u_time_int" << u_time_int << endl;
+								//cout << "buffer:" << buffer << endl;
+								os_my_log_2 << buffer << endl;
+							}
+						}else{
+							//cout << "no-match" << endl;
+						}
 					}
 				}while(!is.eof());
+			}else{
+				;//do nothing
+			}
+			is.close();
+			os_my_log_2.close();
+
+			os_my_log.open(filename_my_log);
+			//open policy file
+			os_policy.open(filename_policy);
+			if(os_policy)
+			{
+				char buffer[MAX_LOG];
+				const char* words="denied";
+				//ޥdeniedåѤ
+				//եƬ˥ݥ󥿤ưƤ
+				os_my_log.seekg(ios::beg);
+				do{
+					os_my_log.getline(buffer, sizeof buffer);
+					if(strstr(buffer,words)){
+						v.push_back(buffer);
+					}
+				}while(!os_my_log.eof());
 				//if there are no denied log
 				if(v.size()==0){
 					os_policy << "Your log is perfect !!" << endl;
@@ -1033,14 +1199,7 @@ void segatex::segatex_show_interface()
 					p_tmp_objclass_end=unique(v_tmp_objclass.begin(),v_tmp_objclass.end());
 
 					os_policy << "####################################################" << endl;
-					os_policy << "##### This file was generated by segatex2.08" << endl;
-					os_policy << "####################################################" << endl;
-					os_policy << "#### $make -f /usr/share/selinux/devel/Makefile" << endl;
-					os_policy << "#### will generate module in this directory." << endl;
-					os_policy << "#### #semodule -i my_policy_by_segatex.pp" << endl;
-					os_policy << "#### will install module to your system."<< endl;
-					os_policy << "#### To check, #semodule -l" << endl;
-					os_policy << "#### To uninstall, #semodule -r my_policy_by_segatex" << endl;
+					os_policy << "##### This file was generated by segatex2.09" << endl;
 					os_policy << "####################################################" << endl;
 					os_policy << "module my_policy_by_segatex 1.0.0;" << endl << endl;
 					os_policy << "require {" << endl;
@@ -1099,7 +1258,7 @@ void segatex::segatex_show_interface()
 						string role_tmp_str= *p_role;
 						os_policy << "	role " << role_tmp_str << ";" << endl;
 					}
-					os_policy << "};" << endl;
+					os_policy << "};" << endl << endl;
 					/*ޥåʤäallowʸФ*/
 					cmatch what;
 					const char* m_snmfp = "(.*):([^[:space:]]*)([:space:])?";
@@ -1181,7 +1340,8 @@ void segatex::segatex_show_interface()
 			}
 			os_policy.close();
 		}
-		is.close();
+		//is.close();
+		os_my_log.close();
 	}
 	is_engine.close();
 	os_policy.close();
